@@ -7,7 +7,7 @@ use Carp;
 use Net::DNS::Resolver;
 use IO::Select;
 
-our $VERSION = '0.200';
+our $VERSION = '0.204';
 
 =head1 NAME
 
@@ -19,9 +19,11 @@ Net::DNSBL::Client - Client code for querying multiple DNSBLs
     my $c = Net::DNSBL::Client->new({ timeout => 3 });
 
     $c->query_ip('127.0.0.2', [
-        { domain => 'simple.dnsbl.tld' },
-        { domain => 'masked.dnsbl.tld', type => 'mask', data => '0.0.0.255' }
-    ]);
+            { domain => 'simple.dnsbl.tld' },
+            { domain => 'masked.dnsbl.tld', type => 'mask', data => '0.0.0.255' },
+            { domain => 'txt.dnsbl.tld', type => 'txt' },
+            { domain => 'need-a-key.example.net' }],
+        { lookup_keys => { 'need-a-key.example.net' => 'my_secret_key' }});
 
     # And later...
     my $answers = $c->get_answers();
@@ -130,11 +132,12 @@ is a hash with the following members:
 =item type
 
 (optional) The type of DNSBL.  Possible values are I<normal>, meaning
-that any returned A record indicates a hit, I<match>, meaning that
-one of the returned A records must exactly match a given IP address, or
-I<mask>, meaning that one of the returned A records must evaluate to non-zero
-when bitwise-ANDed against a given IP address.  If omitted, type defaults
-to I<normal>
+that any returned A record indicates a hit, I<match>, meaning that one
+of the returned A records must exactly match a given IP address,
+I<mask>, meaning that one of the returned A records must evaluate to
+non-zero when bitwise-ANDed against a given IP address, or I<txt>
+meaning that TXT records should be looked up and returned (rather than
+A records)a.  If omitted, type defaults to I<normal>
 
 =item data
 
@@ -151,7 +154,7 @@ It is simply returned back unchanged in the list of hits.
 
 =back
 
-$options, if supplied, is a hash of options.  Currently, two options
+$options, if supplied, is a hash of options.  Currently, three options
 are defined:
 
 =over 4
@@ -169,8 +172,20 @@ hit.  If set to 0 (the default), then the return value from
 I<get_answers()> only returns entries for those DNSBLs that actually
 hit.
 
+=item lookup_keys
+
+This is a hashref of domain_name => key.  Some domains require a secret
+key to be inserted just before the domain name; rather than including
+the key in the domain, you can separate it out with the lookup_keys hash,
+making the returned results more readable.
+
 =back
 
+=item query_domain ( $domain, $dnsbls [, $options])
+
+Similar to query_ip, but considers $domain to be a domain name rather
+than an IP address, and does not reverse the domain.
+
 =item get_answers ( )
 
 This method may only be called while a query is in flight.  It waits
@@ -206,9 +221,10 @@ The data supplied (for normal and mask types)
 
 The userdata as supplied in the I<query_ip()> call
 
-=item actual_hit
+=item actual_hits
 
-The actual A record returned by the lookup that caused a hit.
+Reference to array containing actual A or TXT records returned by the
+lookup that caused a hit.
 
 =item replycode
 
@@ -259,25 +275,41 @@ sub query_is_in_flight
 sub query_ip
 {
 	my ($self, $ipaddr, $dnsbls, $options) = @_;
+	croak('Cannot issue new query while one is in flight') if $self->{in_flight};
+	croak('First argument (ip address) is required')       unless $ipaddr;
+	croak('Second argument (dnsbl list) is required')      unless $dnsbls;
+
+	# Reverse the IP address in preparation for lookups
+	my $revip = $self->_reverse_address($ipaddr);
+
+	return $self->query_domain($revip, $dnsbls, $options);
+}
+
+sub query_domain
+{
+	my ($self, $ip_or_domain, $dnsbls, $options) = @_;
 
 	croak('Cannot issue new query while one is in flight') if $self->{in_flight};
-	croak('First argument (ip address) is required')     unless $ipaddr;
-	croak('Second argument (dnsbl list) is required')    unless $dnsbls;
+	croak('First argument (domain) is required')           unless $ip_or_domain;
+	croak('Second argument (dnsbl list) is required')      unless $dnsbls;
 
-	foreach my $opt qw(early_exit return_all) {
+	foreach my $opt (qw(early_exit return_all)) {
 		if ($options && exists($options->{$opt})) {
 			$self->{$opt} = $options->{$opt};
 		} else {
 			$self->{$opt} = 0;
 		}
 	}
-	# Reverse the IP address in preparation for lookups
-	my $revip = $self->_reverse_address($ipaddr);
 
 	# Build a hash of domains to query.  The key is the domain;
 	# value is an arrayref of type/data pairs
 	$self->{domains} = $self->_build_domains($dnsbls);
-	$self->_send_queries($revip);
+	my $lookup_keys = {};
+	if ($options && exists($options->{lookup_keys}) && ref($options->{lookup_keys}) eq 'HASH') {
+		$lookup_keys = $options->{lookup_keys};
+	}
+
+	$self->_send_queries($ip_or_domain, $lookup_keys);
 }
 
 sub get_answers
@@ -323,14 +355,21 @@ sub _build_domains
 
 sub _send_queries
 {
-	my ($self, $revip) = @_;
+	my ($self, $ip_or_domain, $lookup_keys) = @_;
 
 	$self->{in_flight} = 1;
 	$self->{sel} = IO::Select->new();
 	$self->{sock_to_domain} = {};
 
 	foreach my $domain (keys(%{$self->{domains}})) {
-		my $sock = $self->{resolver}->bgsend("$revip.$domain", 'A');
+		my $lookup_key;
+		if (exists($lookup_keys->{$domain}) && ($lookup_keys->{$domain} ne '')) {
+			$lookup_key = '.' . $lookup_keys->{$domain};
+		} else {
+			$lookup_key = '';
+		}
+		my $sock;
+		$sock = $self->{resolver}->bgsend("$ip_or_domain$lookup_key.$domain", 'ANY');
 		unless ($sock) {
 			die $self->{resolver}->errorstring;
 		}
@@ -386,17 +425,20 @@ sub _process_reply
 
 	my $got_a_hit = 0;
 	foreach my $rr ($pack->answer) {
-		next unless $rr->type eq 'A';
+		next unless ($rr->type eq 'A' || uc($rr->type) eq 'TXT');
 		foreach my $dnsbl (@$entry) {
-			next if $dnsbl->{hit};
+			my $this_rr_hit = 0;
+			next if $dnsbl->{hit} && ($dnsbl->{type} eq 'match');
 			$dnsbl->{replycode} = $rcode;
 			if ($dnsbl->{type} eq 'normal') {
-				$dnsbl->{hit} = 1;
+				next unless $rr->type eq 'A';
+				$this_rr_hit = 1;
 			} elsif ($dnsbl->{type} eq 'match') {
+				next unless $rr->type eq 'A';
 				next unless $rr->address eq $dnsbl->{data};
-				$dnsbl->{hit} = 1;
+				$this_rr_hit = 1;
 			} elsif ($dnsbl->{type} eq 'mask') {
-
+				next unless $rr->type eq 'A';
 				my @quads;
 				# For mask, we can be given an IP mask like
 				# a.b.c.d, or an integer n.  The latter case
@@ -411,11 +453,22 @@ sub _process_reply
 				my $got  = unpack('N',pack('C4', split(/\./,$rr->address)));
 				next unless ($got & $mask);
 
-				$dnsbl->{hit} = 1;
+				$this_rr_hit = 1;
+			} elsif ($dnsbl->{type} eq 'txt') {
+				next unless uc($rr->type) eq 'TXT';
+				$this_rr_hit = 1;
 			}
 
-			if( $dnsbl->{hit} ) {
-				$dnsbl->{actual_hit} = $rr->address;
+			if( $this_rr_hit ) {
+				$dnsbl->{hit} = 1;
+				if( ! $dnsbl->{actual_hits} ) {
+					$dnsbl->{actual_hits} = [];
+				}
+				if ($rr->type eq 'A') {
+					push @{$dnsbl->{actual_hits}}, $rr->address;
+				} else {
+					push @{$dnsbl->{actual_hits}}, $rr->txtdata;
+				}
 				$got_a_hit = 1;
 			}
 		}
